@@ -15,23 +15,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QApplication>
-#include <QBuffer>
+#include <limits>
+#include <set>
+
 #include <QObject>
+#include <QPainter>
+#include <QScroller>
 #include <QTimer>
 
-#include "Cache.h"
 #include "Logging.h"
 #include "MainWindow.h"
-#include "MatrixClient.h"
 #include "RoomInfoListItem.h"
 #include "RoomList.h"
 #include "UserSettingsPage.h"
 #include "Utils.h"
 #include "ui/OverlayModal.h"
 
-RoomList::RoomList(QWidget *parent)
+RoomList::RoomList(QSharedPointer<UserSettings> userSettings, QWidget *parent)
   : QWidget(parent)
+  , settings(userSettings)
 {
         topLayout_ = new QVBoxLayout(this);
         topLayout_->setSpacing(0);
@@ -43,6 +45,8 @@ RoomList::RoomList(QWidget *parent)
         scrollArea_->setWidgetResizable(true);
         scrollArea_->setAlignment(Qt::AlignLeading | Qt::AlignTop | Qt::AlignVCenter);
 
+        QScroller::grabGesture(scrollArea_, QScroller::TouchGesture);
+
         // The scrollbar on macOS will hide itself when not active so it won't interfere
         // with the content.
 #if not defined(Q_OS_MAC)
@@ -61,12 +65,16 @@ RoomList::RoomList(QWidget *parent)
         topLayout_->addWidget(scrollArea_);
 
         connect(this, &RoomList::updateRoomAvatarCb, this, &RoomList::updateRoomAvatar);
+        connect(userSettings.data(),
+                &UserSettings::roomSortingChanged,
+                this,
+                &RoomList::sortRoomsByLastMessage);
 }
 
 void
 RoomList::addRoom(const QString &room_id, const RoomInfo &info)
 {
-        auto room_item = new RoomInfoListItem(room_id, info, scrollArea_);
+        auto room_item = new RoomInfoListItem(room_id, info, settings, scrollArea_);
         room_item->setRoomName(QString::fromStdString(std::move(info.name)));
 
         connect(room_item, &RoomInfoListItem::clicked, this, &RoomList::highlightSelectedRoom);
@@ -86,43 +94,7 @@ RoomList::addRoom(const QString &room_id, const RoomInfo &info)
 void
 RoomList::updateAvatar(const QString &room_id, const QString &url)
 {
-        if (url.isEmpty())
-                return;
-
-        QByteArray savedImgData;
-
-        if (cache::client())
-                savedImgData = cache::client()->image(url);
-
-        if (savedImgData.isEmpty()) {
-                mtx::http::ThumbOpts opts;
-                opts.mxc_url = url.toStdString();
-                http::client()->get_thumbnail(
-                  opts, [room_id, opts, this](const std::string &res, mtx::http::RequestErr err) {
-                          if (err) {
-                                  nhlog::net()->warn(
-                                    "failed to download room avatar: {} {} {}",
-                                    opts.mxc_url,
-                                    mtx::errors::to_string(err->matrix_error.errcode),
-                                    err->matrix_error.error);
-                                  return;
-                          }
-
-                          if (cache::client())
-                                  cache::client()->saveImage(opts.mxc_url, res);
-
-                          auto data = QByteArray(res.data(), res.size());
-                          QPixmap pixmap;
-                          pixmap.loadFromData(data);
-
-                          emit updateRoomAvatarCb(room_id, pixmap);
-                  });
-        } else {
-                QPixmap img;
-                img.loadFromData(savedImgData);
-
-                updateRoomAvatar(room_id, img);
-        }
+        emit updateRoomAvatarCb(room_id, url);
 }
 
 void
@@ -154,6 +126,8 @@ RoomList::updateUnreadMessageCount(const QString &roomid, int count, int highlig
         rooms_[roomid]->updateUnreadMessageCount(count, highlightedCount);
 
         calculateUnreadMessageCount();
+
+        sortRoomsByLastMessage();
 }
 
 void
@@ -193,6 +167,8 @@ RoomList::initialize(const QMap<QString, RoomInfo> &info)
         if (rooms_.empty())
                 return;
 
+        sortRoomsByLastMessage();
+
         auto room = firstRoom();
         if (room.second.isNull())
                 return;
@@ -224,6 +200,9 @@ RoomList::sync(const std::map<QString, RoomInfo> &info)
 {
         for (const auto &room : info)
                 updateRoom(room.first, room.second);
+
+        if (!info.empty())
+                sortRoomsByLastMessage();
 }
 
 void
@@ -252,7 +231,73 @@ RoomList::highlightSelectedRoom(const QString &room_id)
 }
 
 void
-RoomList::updateRoomAvatar(const QString &roomid, const QPixmap &img)
+RoomList::nextRoom()
+{
+        for (int ii = 0; ii < contentsLayout_->count() - 1; ++ii) {
+                auto room = qobject_cast<RoomInfoListItem *>(contentsLayout_->itemAt(ii)->widget());
+
+                if (!room)
+                        continue;
+
+                if (room->roomId() == selectedRoom_) {
+                        auto nextRoom = qobject_cast<RoomInfoListItem *>(
+                          contentsLayout_->itemAt(ii + 1)->widget());
+
+                        // Not a room message.
+                        if (!nextRoom || nextRoom->isInvite())
+                                return;
+
+                        emit roomChanged(nextRoom->roomId());
+                        if (!roomExists(nextRoom->roomId())) {
+                                nhlog::ui()->warn("roomlist: clicked unknown room_id");
+                                return;
+                        }
+
+                        room->setPressedState(false);
+                        nextRoom->setPressedState(true);
+
+                        scrollArea_->ensureWidgetVisible(nextRoom);
+                        selectedRoom_ = nextRoom->roomId();
+                        return;
+                }
+        }
+}
+
+void
+RoomList::previousRoom()
+{
+        for (int ii = 1; ii < contentsLayout_->count(); ++ii) {
+                auto room = qobject_cast<RoomInfoListItem *>(contentsLayout_->itemAt(ii)->widget());
+
+                if (!room)
+                        continue;
+
+                if (room->roomId() == selectedRoom_) {
+                        auto nextRoom = qobject_cast<RoomInfoListItem *>(
+                          contentsLayout_->itemAt(ii - 1)->widget());
+
+                        // Not a room message.
+                        if (!nextRoom || nextRoom->isInvite())
+                                return;
+
+                        emit roomChanged(nextRoom->roomId());
+                        if (!roomExists(nextRoom->roomId())) {
+                                nhlog::ui()->warn("roomlist: clicked unknown room_id");
+                                return;
+                        }
+
+                        room->setPressedState(false);
+                        nextRoom->setPressedState(true);
+
+                        scrollArea_->ensureWidgetVisible(nextRoom);
+                        selectedRoom_ = nextRoom->roomId();
+                        return;
+                }
+        }
+}
+
+void
+RoomList::updateRoomAvatar(const QString &roomid, const QString &img)
 {
         if (!roomExists(roomid)) {
                 nhlog::ui()->warn("avatar update on non-existent room_id: {}",
@@ -260,7 +305,7 @@ RoomList::updateRoomAvatar(const QString &roomid, const QPixmap &img)
                 return;
         }
 
-        rooms_[roomid]->setAvatar(img.toImage());
+        rooms_[roomid]->setAvatar(img);
 
         // Used to inform other widgets for the new image data.
         emit roomAvatarChanged(roomid, img);
@@ -289,28 +334,51 @@ RoomList::updateRoomDescription(const QString &roomid, const DescInfo &info)
         emit sortRoomsByLastMessage();
 }
 
+struct room_sort
+{
+        bool operator()(const RoomInfoListItem *a, const RoomInfoListItem *b) const
+        {
+                // Sort by "importance" (i.e. invites before mentions before
+                // notifs before new events before old events), then secondly
+                // by recency.
+
+                // Checking importance first
+                const auto a_importance = a->calculateImportance();
+                const auto b_importance = b->calculateImportance();
+                if (a_importance != b_importance) {
+                        return a_importance > b_importance;
+                }
+
+                // Now sort by recency
+                // Zero if empty, otherwise the time that the event occured
+                const uint64_t a_recency = a->lastMessageInfo().userid.isEmpty()
+                                             ? 0
+                                             : a->lastMessageInfo().datetime.toMSecsSinceEpoch();
+                const uint64_t b_recency = b->lastMessageInfo().userid.isEmpty()
+                                             ? 0
+                                             : b->lastMessageInfo().datetime.toMSecsSinceEpoch();
+                return a_recency > b_recency;
+        }
+};
+
 void
 RoomList::sortRoomsByLastMessage()
 {
         isSortPending_ = false;
 
-        std::multimap<uint64_t, RoomInfoListItem *, std::greater<uint64_t>> times;
+        std::multiset<RoomInfoListItem *, room_sort> times;
 
         for (int ii = 0; ii < contentsLayout_->count(); ++ii) {
                 auto room = qobject_cast<RoomInfoListItem *>(contentsLayout_->itemAt(ii)->widget());
 
                 if (!room)
                         continue;
-
-                // Not a room message.
-                if (room->lastMessageInfo().userid.isEmpty())
-                        times.emplace(0, room);
                 else
-                        times.emplace(room->lastMessageInfo().datetime.toMSecsSinceEpoch(), room);
+                        times.insert(room);
         }
 
         for (auto it = times.cbegin(); it != times.cend(); ++it) {
-                const auto roomWidget   = it->second;
+                const auto roomWidget   = *it;
                 const auto currentIndex = contentsLayout_->indexOf(roomWidget);
                 const auto newIndex     = std::distance(times.cbegin(), it);
 
@@ -427,7 +495,7 @@ RoomList::updateRoom(const QString &room_id, const RoomInfo &info)
 void
 RoomList::addInvitedRoom(const QString &room_id, const RoomInfo &info)
 {
-        auto room_item = new RoomInfoListItem(room_id, info, scrollArea_);
+        auto room_item = new RoomInfoListItem(room_id, info, settings, scrollArea_);
 
         connect(room_item, &RoomInfoListItem::acceptInvite, this, &RoomList::acceptInvite);
         connect(room_item, &RoomInfoListItem::declineInvite, this, &RoomList::declineInvite);
@@ -443,13 +511,16 @@ RoomList::addInvitedRoom(const QString &room_id, const RoomInfo &info)
 std::pair<QString, QSharedPointer<RoomInfoListItem>>
 RoomList::firstRoom() const
 {
-        auto firstRoom = rooms_.begin();
+        for (int i = 0; i < contentsLayout_->count(); i++) {
+                auto item = qobject_cast<RoomInfoListItem *>(contentsLayout_->itemAt(i)->widget());
 
-        while (firstRoom->second.isNull() && firstRoom != rooms_.end())
-                firstRoom++;
+                if (item) {
+                        return std::pair<QString, QSharedPointer<RoomInfoListItem>>(
+                          item->roomId(), rooms_.at(item->roomId()));
+                }
+        }
 
-        return std::pair<QString, QSharedPointer<RoomInfoListItem>>(firstRoom->first,
-                                                                    firstRoom->second);
+        return {};
 }
 
 void
