@@ -18,9 +18,12 @@
 #pragma once
 
 #include <QComboBox>
+#include <QFontDatabase>
+#include <QFormLayout>
 #include <QFrame>
 #include <QLabel>
 #include <QLayout>
+#include <QProcessEnvironment>
 #include <QSharedPointer>
 #include <QWidget>
 
@@ -53,11 +56,9 @@ public:
                 save();
         }
 
-        void setRoomOrdering(bool state)
-        {
-                isOrderingEnabled_ = state;
-                save();
-        }
+        void setFontSize(double size);
+        void setFontFamily(QString family);
+        void setEmojiFontFamily(QString family);
 
         void setGroupView(bool state)
         {
@@ -68,6 +69,12 @@ public:
                 save();
         }
 
+        void setMarkdownEnabled(bool state)
+        {
+                isMarkdownEnabled_ = state;
+                save();
+        }
+
         void setReadReceipts(bool state)
         {
                 isReadReceiptsEnabled_ = state;
@@ -80,33 +87,77 @@ public:
                 save();
         }
 
+        void setSortByImportance(bool state)
+        {
+                sortByImportance_ = state;
+                emit roomSortingChanged();
+        }
+
+        void setButtonsInTimeline(bool state)
+        {
+                isButtonsInTimelineEnabled_ = state;
+                save();
+        }
+
         void setDesktopNotifications(bool state)
         {
                 hasDesktopNotifications_ = state;
                 save();
         }
 
-        QString theme() const { return !theme_.isEmpty() ? theme_ : "light"; }
+        void setAvatarCircles(bool state)
+        {
+                avatarCircles_ = state;
+                save();
+        }
+
+        void setDecryptSidebar(bool state)
+        {
+                decryptSidebar_ = state;
+                save();
+        }
+
+        QString theme() const { return !theme_.isEmpty() ? theme_ : defaultTheme_; }
         bool isTrayEnabled() const { return isTrayEnabled_; }
         bool isStartInTrayEnabled() const { return isStartInTrayEnabled_; }
-        bool isOrderingEnabled() const { return isOrderingEnabled_; }
         bool isGroupViewEnabled() const { return isGroupViewEnabled_; }
+        bool isAvatarCirclesEnabled() const { return avatarCircles_; }
+        bool isDecryptSidebarEnabled() const { return decryptSidebar_; }
+        bool isMarkdownEnabled() const { return isMarkdownEnabled_; }
         bool isTypingNotificationsEnabled() const { return isTypingNotificationsEnabled_; }
+        bool isSortByImportanceEnabled() const { return sortByImportance_; }
+        bool isButtonsInTimelineEnabled() const { return isButtonsInTimelineEnabled_; }
         bool isReadReceiptsEnabled() const { return isReadReceiptsEnabled_; }
         bool hasDesktopNotifications() const { return hasDesktopNotifications_; }
+        double fontSize() const { return baseFontSize_; }
+        QString font() const { return font_; }
+        QString emojiFont() const { return emojiFont_; }
 
 signals:
         void groupViewStateChanged(bool state);
+        void roomSortingChanged();
 
 private:
+        // Default to system theme if QT_QPA_PLATFORMTHEME var is set.
+        QString defaultTheme_ =
+          QProcessEnvironment::systemEnvironment().value("QT_QPA_PLATFORMTHEME", "").isEmpty()
+            ? "light"
+            : "system";
         QString theme_;
         bool isTrayEnabled_;
         bool isStartInTrayEnabled_;
-        bool isOrderingEnabled_;
         bool isGroupViewEnabled_;
+        bool isMarkdownEnabled_;
         bool isTypingNotificationsEnabled_;
+        bool sortByImportance_;
+        bool isButtonsInTimelineEnabled_;
         bool isReadReceiptsEnabled_;
         bool hasDesktopNotifications_;
+        bool avatarCircles_;
+        bool decryptSidebar_;
+        double baseFontSize_;
+        QString font_;
+        QString emojiFont_;
 };
 
 class HorizontalLine : public QFrame
@@ -122,45 +173,50 @@ class UserSettingsPage : public QWidget
         Q_OBJECT
 
 public:
-        UserSettingsPage(QSharedPointer<UserSettings> settings, QWidget *parent = 0);
+        UserSettingsPage(QSharedPointer<UserSettings> settings, QWidget *parent = nullptr);
 
 protected:
         void showEvent(QShowEvent *event) override;
-        void resizeEvent(QResizeEvent *event) override;
         void paintEvent(QPaintEvent *event) override;
 
 signals:
         void moveBack();
         void trayOptionChanged(bool value);
+        void themeChanged();
+        void decryptSidebarChanged();
 
 private slots:
         void importSessionKeys();
         void exportSessionKeys();
 
 private:
-        void restoreThemeCombo() const;
-        void restoreScaleFactor() const;
-
         // Layouts
         QVBoxLayout *topLayout_;
-        QVBoxLayout *mainLayout_;
         QHBoxLayout *topBarLayout_;
+        QFormLayout *formLayout_;
 
         // Shared settings object.
         QSharedPointer<UserSettings> settings_;
 
         Toggle *trayToggle_;
         Toggle *startInTrayToggle_;
-        Toggle *roomOrderToggle_;
         Toggle *groupViewToggle_;
+        Toggle *timelineButtonsToggle_;
         Toggle *typingNotifications_;
+        Toggle *sortByImportance_;
         Toggle *readReceipts_;
+        Toggle *markdownEnabled_;
         Toggle *desktopNotifications_;
+        Toggle *avatarCircles_;
+        Toggle *decryptSidebar_;
         QLabel *deviceFingerprintValue_;
         QLabel *deviceIdValue_;
 
         QComboBox *themeCombo_;
         QComboBox *scaleFactorCombo_;
+        QComboBox *fontSizeCombo_;
+        QComboBox *fontSelectionCombo_;
+        QComboBox *emojiFontSelectionCombo_;
 
         int sideMargin_ = 0;
 };
